package java.util;

import java.security.PermissionCollection;
import java.security.ProtectionDomain;
import java.security.AccessControlContext;
import java.util.concurrent.ConcurrentHashMap;
import java.time.Instant;
import java.security.PrivilegedActionException;
import java.io.OptionalDataException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import sun.util.locale.provider.CalendarDataUtility;
import java.text.DateFormatSymbols;
import sun.util.calendar.ZoneInfo;
import java.text.DateFormat;
import sun.util.BuddhistCalendar;
import sun.util.locale.provider.LocaleProviderAdapter;
import sun.util.spi.CalendarProvider;
import java.util.concurrent.ConcurrentMap;
import java.io.Serializable;

public abstract class Calendar implements Serializable, Cloneable, Comparable<Calendar>
{
    public static final int ERA = 0;
    public static final int YEAR = 1;
    public static final int MONTH = 2;
    public static final int WEEK_OF_YEAR = 3;
    public static final int WEEK_OF_MONTH = 4;
    public static final int DATE = 5;
    public static final int DAY_OF_MONTH = 5;
    public static final int DAY_OF_YEAR = 6;
    public static final int DAY_OF_WEEK = 7;
    public static final int DAY_OF_WEEK_IN_MONTH = 8;
    public static final int AM_PM = 9;
    public static final int HOUR = 10;
    public static final int HOUR_OF_DAY = 11;
    public static final int MINUTE = 12;
    public static final int SECOND = 13;
    public static final int MILLISECOND = 14;
    public static final int ZONE_OFFSET = 15;
    public static final int DST_OFFSET = 16;
    public static final int FIELD_COUNT = 17;
    public static final int SUNDAY = 1;
    public static final int MONDAY = 2;
    public static final int TUESDAY = 3;
    public static final int WEDNESDAY = 4;
    public static final int THURSDAY = 5;
    public static final int FRIDAY = 6;
    public static final int SATURDAY = 7;
    public static final int JANUARY = 0;
    public static final int FEBRUARY = 1;
    public static final int MARCH = 2;
    public static final int APRIL = 3;
    public static final int MAY = 4;
    public static final int JUNE = 5;
    public static final int JULY = 6;
    public static final int AUGUST = 7;
    public static final int SEPTEMBER = 8;
    public static final int OCTOBER = 9;
    public static final int NOVEMBER = 10;
    public static final int DECEMBER = 11;
    public static final int UNDECIMBER = 12;
    public static final int AM = 0;
    public static final int PM = 1;
    public static final int ALL_STYLES = 0;
    static final int STANDALONE_MASK = 32768;
    public static final int SHORT = 1;
    public static final int LONG = 2;
    public static final int NARROW_FORMAT = 4;
    public static final int NARROW_STANDALONE = 32772;
    public static final int SHORT_FORMAT = 1;
    public static final int LONG_FORMAT = 2;
    public static final int SHORT_STANDALONE = 32769;
    public static final int LONG_STANDALONE = 32770;
    protected int[] fields;
    protected boolean[] isSet;
    private transient int[] stamp;
    protected long time;
    protected boolean isTimeSet;
    protected boolean areFieldsSet;
    transient boolean areAllFieldsSet;
    private boolean lenient;
    private TimeZone zone;
    private transient boolean sharedZone;
    private int firstDayOfWeek;
    private int minimalDaysInFirstWeek;
    private static final ConcurrentMap<Locale, int[]> cachedLocaleData;
    private static final int UNSET = 0;
    private static final int COMPUTED = 1;
    private static final int MINIMUM_USER_STAMP = 2;
    static final int ALL_FIELDS = 131071;
    private int nextStamp;
    static final int currentSerialVersion = 1;
    private int serialVersionOnStream;
    static final long serialVersionUID = -1807547505821590642L;
    static final int ERA_MASK = 1;
    static final int YEAR_MASK = 2;
    static final int MONTH_MASK = 4;
    static final int WEEK_OF_YEAR_MASK = 8;
    static final int WEEK_OF_MONTH_MASK = 16;
    static final int DAY_OF_MONTH_MASK = 32;
    static final int DATE_MASK = 32;
    static final int DAY_OF_YEAR_MASK = 64;
    static final int DAY_OF_WEEK_MASK = 128;
    static final int DAY_OF_WEEK_IN_MONTH_MASK = 256;
    static final int AM_PM_MASK = 512;
    static final int HOUR_MASK = 1024;
    static final int HOUR_OF_DAY_MASK = 2048;
    static final int MINUTE_MASK = 4096;
    static final int SECOND_MASK = 8192;
    static final int MILLISECOND_MASK = 16384;
    static final int ZONE_OFFSET_MASK = 32768;
    static final int DST_OFFSET_MASK = 65536;
    private static final String[] FIELD_NAME;
    
    protected Calendar() {
        this(TimeZone.getDefaultRef(), Locale.getDefault(Locale.Category.FORMAT));
        this.sharedZone = true;
    }
    
    protected Calendar(TimeZone zone, Locale weekCountData) {
        this.lenient = true;
        this.sharedZone = false;
        this.nextStamp = 2;
        this.serialVersionOnStream = 1;
        this.fields = new int[17];
        this.isSet = new boolean[17];
        this.stamp = new int[17];
        this.zone = zone;
        this.setWeekCountData(weekCountData);
    }
    
    public static Calendar getInstance() {
        return createCalendar(TimeZone.getDefault(), Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public static Calendar getInstance(TimeZone timeZone) {
        return createCalendar(timeZone, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public static Calendar getInstance(Locale locale) {
        return createCalendar(TimeZone.getDefault(), locale);
    }
    
    public static Calendar getInstance(TimeZone timeZone, Locale locale) {
        return createCalendar(timeZone, locale);
    }
    
    private static Calendar createCalendar(TimeZone timeZone, Locale aLocale) {
        CalendarProvider calendarProvider = LocaleProviderAdapter.getAdapter(CalendarProvider.class, aLocale).getCalendarProvider();
        if (calendarProvider != null) {
            try {
                return calendarProvider.getInstance(timeZone, aLocale);
            }
            catch (IllegalArgumentException ex) {}
        }
        Object o = null;
        if (aLocale.hasExtensions()) {
            String unicodeLocaleType = aLocale.getUnicodeLocaleType("ca");
            if (unicodeLocaleType != null) {
                int n = -1;
                switch (unicodeLocaleType.hashCode()) {
                    case -1581060683: {
                        if (unicodeLocaleType.equals("buddhist")) {
                            n = 0;
                            break;
                        }
                        break;
                    }
                    case -752730191: {
                        if (unicodeLocaleType.equals("japanese")) {
                            n = 1;
                            break;
                        }
                        break;
                    }
                    case 283776265: {
                        if (unicodeLocaleType.equals("gregory")) {
                            n = 2;
                            break;
                        }
                        break;
                    }
                }
                switch (n) {
                    case 0: {
                        o = new BuddhistCalendar(timeZone, aLocale);
                        break;
                    }
                    case 1: {
                        o = new JapaneseImperialCalendar(timeZone, aLocale);
                        break;
                    }
                    case 2: {
                        o = new GregorianCalendar(timeZone, aLocale);
                        break;
                    }
                }
            }
        }
        if (o == null) {
            if (aLocale.getLanguage() == "th" && aLocale.getCountry() == "TH") {
                o = new BuddhistCalendar(timeZone, aLocale);
            }
            else if (aLocale.getVariant() == "JP" && aLocale.getLanguage() == "ja" && aLocale.getCountry() == "JP") {
                o = new JapaneseImperialCalendar(timeZone, aLocale);
            }
            else {
                o = new GregorianCalendar(timeZone, aLocale);
            }
        }
        return o;
    }
    
    public static synchronized Locale[] getAvailableLocales() {
        return DateFormat.getAvailableLocales();
    }
    
    protected abstract void computeTime();
    
    protected abstract void computeFields();
    
    public final Date getTime() {
        return new Date(this.getTimeInMillis());
    }
    
    public final void setTime(Date date) {
        this.setTimeInMillis(date.getTime());
    }
    
    public long getTimeInMillis() {
        if (!this.isTimeSet) {
            this.updateTime();
        }
        return this.time;
    }
    
    public void setTimeInMillis(long time) {
        if (this.time == time && this.isTimeSet && this.areFieldsSet && this.areAllFieldsSet && this.zone instanceof ZoneInfo && !((ZoneInfo)this.zone).isDirty()) {
            return;
        }
        this.time = time;
        this.isTimeSet = true;
        this.areFieldsSet = false;
        this.computeFields();
        boolean b = true;
        this.areFieldsSet = b;
        this.areAllFieldsSet = b;
        return;
    }
    
    public int get(int n) {
        this.complete();
        return this.internalGet(n);
    }
    
    protected final int internalGet(int n) {
        return this.fields[n];
    }
    
    final void internalSet(int n, int n2) {
        this.fields[n] = n2;
    }
    
    public void set(int n, int n2) {
        if (this.areFieldsSet && !this.areAllFieldsSet) {
            this.computeFields();
        }
        this.internalSet(n, n2);
        this.isTimeSet = false;
        this.areFieldsSet = false;
        this.isSet[n] = true;
        this.stamp[n] = this.nextStamp++;
        if (this.nextStamp == Integer.MAX_VALUE) {
            this.adjustStamp();
        }
    }
    
    public final void set(int n, int n2, int n3) {
        this.set(1, n);
        this.set(2, n2);
        this.set(5, n3);
    }
    
    public final void set(int n, int n2, int n3, int n4, int n5) {
        this.set(1, n);
        this.set(2, n2);
        this.set(5, n3);
        this.set(11, n4);
        this.set(12, n5);
    }
    
    public final void set(int n, int n2, int n3, int n4, int n5, int n6) {
        this.set(1, n);
        this.set(2, n2);
        this.set(5, n3);
        this.set(11, n4);
        this.set(12, n5);
        this.set(13, n6);
    }
    
    public final void clear() {
        for (int i = 0; i < this.fields.length; this.isSet[i++] = false) {
            this.stamp[i] = (this.fields[i] = 0);
        }
        boolean b = false;
        this.areFieldsSet = b;
        this.areAllFieldsSet = b;
        this.isTimeSet = false;
    }
    
    public final void clear(int n) {
        this.fields[n] = 0;
        this.stamp[n] = 0;
        this.isSet[n] = false;
        boolean b = false;
        this.areFieldsSet = b;
        this.areAllFieldsSet = b;
        this.isTimeSet = false;
    }
    
    public final boolean isSet(int n) {
        return this.stamp[n] != 0;
    }
    
    public String getDisplayName(int field, int style, Locale locale) {
        if (!this.checkDisplayNameParams(field, style, 1, 4, locale, 645)) {
            return null;
        }
        String calendarType = this.getCalendarType();
        int value = this.get(field);
        if (this.isStandaloneStyle(style) || this.isNarrowFormatStyle(style)) {
            String s = CalendarDataUtility.retrieveFieldValueName(calendarType, field, value, style, locale);
            if (s == null) {
                if (this.isNarrowFormatStyle(style)) {
                    s = CalendarDataUtility.retrieveFieldValueName(calendarType, field, value, this.toStandaloneStyle(style), locale);
                }
                else if (this.isStandaloneStyle(style)) {
                    s = CalendarDataUtility.retrieveFieldValueName(calendarType, field, value, this.getBaseStyle(style), locale);
                }
            }
            return s;
        }
        else {
            String[] fieldStrings = this.getFieldStrings(field, style, DateFormatSymbols.getInstance(locale));
            if (fieldStrings != null && value < fieldStrings.length) {
                return fieldStrings[value];
            }
            return null;
        }
    }
    
    public Map<String, Integer> getDisplayNames(int field, int style, Locale locale) {
        if (!this.checkDisplayNameParams(field, style, 0, 4, locale, 645)) {
            return null;
        }
        String calendarType = this.getCalendarType();
        if (style == 0 || this.isStandaloneStyle(style) || this.isNarrowFormatStyle(style)) {
            Map<String, Integer> map = CalendarDataUtility.retrieveFieldValueNames(calendarType, field, style, locale);
            if (map == null) {
                if (this.isNarrowFormatStyle(style)) {
                    map = CalendarDataUtility.retrieveFieldValueNames(calendarType, field, this.toStandaloneStyle(style), locale);
                }
                else if (style != 0) {
                    map = CalendarDataUtility.retrieveFieldValueNames(calendarType, field, this.getBaseStyle(style), locale);
                }
            }
            return map;
        }
        else {
            return this.getDisplayNamesImpl(field, style, locale);
        }
    }
    
    private Map<String, Integer> getDisplayNamesImpl(int n, int n2, Locale locale) {
        String[] fieldStrings = this.getFieldStrings(n, n2, DateFormatSymbols.getInstance(locale));
        if (fieldStrings != null) {
            HashMap<Object, Integer> hashMap = new HashMap();
            for (int i = 0; i < fieldStrings.length; ++i) {
                if (fieldStrings[i].length() != 0) {
                    hashMap.put(fieldStrings[i], Integer.valueOf(i));
                }
            }
            return hashMap;
        }
        return null;
    }
    
    boolean checkDisplayNameParams(int n, int n2, int n3, int n4, Locale locale, int n5) {
        int baseStyle = this.getBaseStyle(n2);
        if (n < 0 || n >= this.fields.length || baseStyle < n3 || baseStyle > n4) {
            throw new IllegalArgumentException();
        }
        if (locale == null) {
            throw new NullPointerException();
        }
        return isFieldSet(n5, n);
    }
    
    private String[] getFieldStrings(int n, int n2, DateFormatSymbols dateFormatSymbols) {
        int baseStyle = this.getBaseStyle(n2);
        if (baseStyle == 4) {
            return null;
        }
        String[] array = null;
        switch (n) {
            case 0: {
                array = dateFormatSymbols.getEras();
                break;
            }
            case 2: {
                array = ((baseStyle == 2) ? dateFormatSymbols.getMonths() : dateFormatSymbols.getShortMonths());
                break;
            }
            case 7: {
                array = ((baseStyle == 2) ? dateFormatSymbols.getWeekdays() : dateFormatSymbols.getShortWeekdays());
                break;
            }
            case 9: {
                array = dateFormatSymbols.getAmPmStrings();
                break;
            }
        }
        return array;
    }
    
    protected void complete() {
        if (!this.isTimeSet) {
            this.updateTime();
        }
        if (!this.areFieldsSet || !this.areAllFieldsSet) {
            this.computeFields();
            boolean b = true;
            this.areFieldsSet = b;
            this.areAllFieldsSet = b;
        }
    }
    
    final boolean isExternallySet(int n) {
        return this.stamp[n] >= 2;
    }
    
    final int getSetStateFields() {
        int n = 0;
        for (int i = 0; i < this.fields.length; ++i) {
            if (this.stamp[i] != 0) {
                n |= 1 << i;
            }
        }
        return n;
    }
    
    final void setFieldsComputed(int n) {
        if (n == 131071) {
            for (int j = 0; j < this.fields.length; ++j) {
                this.stamp[j] = 1;
                this.isSet[j] = true;
            }
            boolean b = true;
            this.areAllFieldsSet = b;
            this.areFieldsSet = b;
        }
        else {
            for (int i = 0; i < this.fields.length; n >>>= 1, ++i) {
                if ((n & 0x1) == 0x1) {
                    this.stamp[i] = 1;
                    this.isSet[i] = true;
                }
                else if (this.areAllFieldsSet && !this.isSet[i]) {
                    this.areAllFieldsSet = false;
                }
            }
        }
    }
    
    final void setFieldsNormalized(int n) {
        if (n != 131071) {
            for (int i = 0; i < this.fields.length; n >>= 1, ++i) {
                if ((n & 0x1) == 0x0) {
                    this.stamp[i] = (this.fields[i] = 0);
                    this.isSet[i] = false;
                }
            }
        }
        this.areFieldsSet = true;
        this.areAllFieldsSet = false;
    }
    
    final boolean isPartiallyNormalized() {
        return this.areFieldsSet && !this.areAllFieldsSet;
    }
    
    final boolean isFullyNormalized() {
        return this.areFieldsSet && this.areAllFieldsSet;
    }
    
    final void setUnnormalized() {
        boolean b = false;
        this.areAllFieldsSet = b;
        this.areFieldsSet = b;
    }
    
    static boolean isFieldSet(int n, int n2) {
        return (n & 1 << n2) != 0x0;
    }
    
    final int selectFields() {
        int n = 2;
        if (this.stamp[0] != 0) {
            n |= 0x1;
        }
        int b = this.stamp[7];
        int n2 = this.stamp[2];
        int n3 = this.stamp[5];
        int aggregateStamp = aggregateStamp(this.stamp[4], b);
        int b2 = aggregateStamp(this.stamp[8], b);
        int n4 = this.stamp[6];
        int aggregateStamp2 = aggregateStamp(this.stamp[3], b);
        int max = n3;
        if (aggregateStamp > max) {
            max = aggregateStamp;
        }
        if (b2 > max) {
            max = b2;
        }
        if (n4 > max) {
            max = n4;
        }
        if (aggregateStamp2 > max) {
            max = aggregateStamp2;
        }
        if (max == 0) {
            aggregateStamp = this.stamp[4];
            b2 = Math.max(this.stamp[8], b);
            aggregateStamp2 = this.stamp[3];
            max = Math.max(Math.max(aggregateStamp, b2), aggregateStamp2);
            if (max == 0) {
                n3 = (max = n2);
            }
        }
        int n5;
        if (max == n3 || (max == aggregateStamp && this.stamp[4] >= this.stamp[3]) || (max == b2 && this.stamp[8] >= this.stamp[3])) {
            n5 = (n | 0x4);
            if (max == n3) {
                n5 |= 0x20;
            }
            else {
                assert max == b2;
                if (b != 0) {
                    n5 |= 0x80;
                }
                if (aggregateStamp == b2) {
                    if (this.stamp[4] >= this.stamp[8]) {
                        n5 |= 0x10;
                    }
                    else {
                        n5 |= 0x100;
                    }
                }
                else if (max == aggregateStamp) {
                    n5 |= 0x10;
                }
                else {
                    assert max == b2;
                    if (this.stamp[8] != 0) {
                        n5 |= 0x100;
                    }
                }
            }
        }
        else {
            assert max == 0;
            if (max == n4) {
                n5 = (n | 0x40);
            }
            else {
                assert max == aggregateStamp2;
                if (b != 0) {
                    n |= 0x80;
                }
                n5 = (n | 0x8);
            }
        }
        int n6 = this.stamp[11];
        int aggregateStamp3 = aggregateStamp(this.stamp[10], this.stamp[9]);
        int max2 = (aggregateStamp3 > n6) ? aggregateStamp3 : n6;
        if (max2 == 0) {
            max2 = Math.max(this.stamp[10], this.stamp[9]);
        }
        if (max2 != 0) {
            if (max2 == n6) {
                n5 |= 0x800;
            }
            else {
                n5 |= 0x400;
                if (this.stamp[9] != 0) {
                    n5 |= 0x200;
                }
            }
        }
        if (this.stamp[12] != 0) {
            n5 |= 0x1000;
        }
        if (this.stamp[13] != 0) {
            n5 |= 0x2000;
        }
        if (this.stamp[14] != 0) {
            n5 |= 0x4000;
        }
        if (this.stamp[15] >= 2) {
            n5 |= 0x8000;
        }
        if (this.stamp[16] >= 2) {
            n5 |= 0x10000;
        }
        return n5;
    }
    
    int getBaseStyle(int n) {
        return n & 0xFFFF7FFF;
    }
    
    private int toStandaloneStyle(int n) {
        return n | 0x8000;
    }
    
    private boolean isStandaloneStyle(int n) {
        return (n & 0x8000) != 0x0;
    }
    
    private boolean isNarrowStyle(int n) {
        return n == 4 || n == 32772;
    }
    
    private boolean isNarrowFormatStyle(int n) {
        return n == 4;
    }
    
    private static int aggregateStamp(int n, int n2) {
        if (n == 0 || n2 == 0) {
            return 0;
        }
        return (n > n2) ? n : n2;
    }
    
    public static Set<String> getAvailableCalendarTypes() {
        return Calendar.AvailableCalendarTypes.SET;
    }
    
    public String getCalendarType() {
        return this.getClass().getName();
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        try {
            Calendar calendar = (Calendar)o;
            return this.compareTo(getMillisOf(calendar)) == 0 && this.lenient == calendar.lenient && this.firstDayOfWeek == calendar.firstDayOfWeek && this.minimalDaysInFirstWeek == calendar.minimalDaysInFirstWeek && ((this.zone instanceof ZoneInfo) ? this.zone.equals(calendar.zone) : this.zone.equals(calendar.getTimeZone()));
        }
        catch (Exception ex) {
            return false;
        }
    }
    
    @Override
    public int hashCode() {
        int n = (this.lenient ? 1 : 0) | this.firstDayOfWeek << 1 | this.minimalDaysInFirstWeek << 4 | this.zone.hashCode() << 7;
        long millis = getMillisOf(this);
        return (int)millis ^ (int)(millis >> 32) ^ n;
    }
    
    public boolean before(Object o) {
        return o instanceof Calendar && this.compareTo((Calendar)o) < 0;
    }
    
    public boolean after(Object o) {
        return o instanceof Calendar && this.compareTo((Calendar)o) > 0;
    }
    
    @Override
    public int compareTo(Calendar calendar) {
        return this.compareTo(getMillisOf(calendar));
    }
    
    public abstract void add(int p0, int p1);
    
    public abstract void roll(int p0, boolean p1);
    
    public void roll(int n, int i) {
        while (i > 0) {
            this.roll(n, true);
            --i;
        }
        while (i < 0) {
            this.roll(n, false);
            ++i;
        }
    }
    
    public void setTimeZone(TimeZone zone) {
        this.zone = zone;
        this.sharedZone = false;
        boolean b = false;
        this.areFieldsSet = b;
        this.areAllFieldsSet = b;
    }
    
    public TimeZone getTimeZone() {
        if (this.sharedZone) {
            this.zone = (TimeZone)this.zone.clone();
            this.sharedZone = false;
        }
        return this.zone;
    }
    
    TimeZone getZone() {
        return this.zone;
    }
    
    void setZoneShared(boolean sharedZone) {
        this.sharedZone = sharedZone;
    }
    
    public void setLenient(boolean lenient) {
        this.lenient = lenient;
    }
    
    public boolean isLenient() {
        return this.lenient;
    }
    
    public void setFirstDayOfWeek(int firstDayOfWeek) {
        if (this.firstDayOfWeek == firstDayOfWeek) {
            return;
        }
        this.firstDayOfWeek = firstDayOfWeek;
        this.invalidateWeekFields();
        return;
    }
    
    public int getFirstDayOfWeek() {
        return this.firstDayOfWeek;
    }
    
    public void setMinimalDaysInFirstWeek(int minimalDaysInFirstWeek) {
        if (this.minimalDaysInFirstWeek == minimalDaysInFirstWeek) {
            return;
        }
        this.minimalDaysInFirstWeek = minimalDaysInFirstWeek;
        this.invalidateWeekFields();
        return;
    }
    
    public int getMinimalDaysInFirstWeek() {
        return this.minimalDaysInFirstWeek;
    }
    
    public boolean isWeekDateSupported() {
        return false;
    }
    
    public int getWeekYear() {
        throw new UnsupportedOperationException();
    }
    
    public void setWeekDate(int n, int n2, int n3) {
        throw new UnsupportedOperationException();
    }
    
    public int getWeeksInWeekYear() {
        throw new UnsupportedOperationException();
    }
    
    public abstract int getMinimum(int p0);
    
    public abstract int getMaximum(int p0);
    
    public abstract int getGreatestMinimum(int p0);
    
    public abstract int getLeastMaximum(int p0);
    
    public int getActualMinimum(int n) {
        int greatestMinimum = this.getGreatestMinimum(n);
        int minimum = this.getMinimum(n);
        if (greatestMinimum == minimum) {
            return greatestMinimum;
        }
        Calendar calendar = (Calendar)this.clone();
        calendar.setLenient(true);
        do {
            calendar.set(n, greatestMinimum);
            if (calendar.get(n) != greatestMinimum) {
                break;
            }
        } while (--greatestMinimum >= minimum);
        return greatestMinimum;
    }
    
    public int getActualMaximum(int n) {
        int leastMaximum = this.getLeastMaximum(n);
        int maximum = this.getMaximum(n);
        if (leastMaximum == maximum) {
            return leastMaximum;
        }
        Calendar calendar = (Calendar)this.clone();
        calendar.setLenient(true);
        if (n == 3 || n == 4) {
            calendar.set(7, this.firstDayOfWeek);
        }
        do {
            calendar.set(n, leastMaximum);
            if (calendar.get(n) != leastMaximum) {
                break;
            }
        } while (++leastMaximum <= maximum);
        return leastMaximum;
    }
    
    public Object clone() {
        try {
            Calendar calendar = (Calendar)super.clone();
            calendar.fields = new int[17];
            calendar.isSet = new boolean[17];
            calendar.stamp = new int[17];
            for (int i = 0; i < 17; ++i) {
                calendar.fields[i] = this.fields[i];
                calendar.stamp[i] = this.stamp[i];
                calendar.isSet[i] = this.isSet[i];
            }
            calendar.zone = (TimeZone)this.zone.clone();
            return calendar;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    static String getFieldName(int n) {
        return Calendar.FIELD_NAME[n];
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(800);
        sb.append(this.getClass().getName()).append('[');
        appendValue(sb, "time", this.isTimeSet, this.time);
        sb.append(",areFieldsSet=").append(this.areFieldsSet);
        sb.append(",areAllFieldsSet=").append(this.areAllFieldsSet);
        sb.append(",lenient=").append(this.lenient);
        sb.append(",zone=").append(this.zone);
        appendValue(sb, ",firstDayOfWeek", true, (long)this.firstDayOfWeek);
        appendValue(sb, ",minimalDaysInFirstWeek", true, (long)this.minimalDaysInFirstWeek);
        for (int i = 0; i < 17; ++i) {
            sb.append(',');
            appendValue(sb, Calendar.FIELD_NAME[i], this.isSet(i), (long)this.fields[i]);
        }
        sb.append(']');
        return sb.toString();
    }
    
    private static void appendValue(StringBuilder sb, String str, boolean b, long lng) {
        sb.append(str).append('=');
        if (b) {
            sb.append(lng);
        }
        else {
            sb.append('?');
        }
    }
    
    private void setWeekCountData(Locale locale) {
        int[] array = (int[])Calendar.cachedLocaleData.get(locale);
        if (array == null) {
            array = new int[] { CalendarDataUtility.retrieveFirstDayOfWeek(locale), CalendarDataUtility.retrieveMinimalDaysInFirstWeek(locale) };
            Calendar.cachedLocaleData.putIfAbsent(locale, array);
        }
        this.firstDayOfWeek = array[0];
        this.minimalDaysInFirstWeek = array[1];
    }
    
    private void updateTime() {
        this.computeTime();
        this.isTimeSet = true;
    }
    
    private int compareTo(long n) {
        long millis = getMillisOf(this);
        return (millis > n) ? 1 : ((millis == n) ? 0 : -1);
    }
    
    private static long getMillisOf(Calendar calendar) {
        if (calendar.isTimeSet) {
            return calendar.time;
        }
        Calendar calendar2 = (Calendar)calendar.clone();
        calendar2.setLenient(true);
        return calendar2.getTimeInMillis();
    }
    
    private void adjustStamp() {
        int n = 2;
        int nextStamp = 2;
        while (true) {
            int n2 = Integer.MAX_VALUE;
            for (int i = 0; i < this.stamp.length; ++i) {
                int n3 = this.stamp[i];
                if (n3 >= nextStamp && n2 > n3) {
                    n2 = n3;
                }
                if (n < n3) {
                    n = n3;
                }
            }
            if (n != n2 && n2 == Integer.MAX_VALUE) {
                break;
            }
            for (int j = 0; j < this.stamp.length; ++j) {
                if (this.stamp[j] == n2) {
                    this.stamp[j] = nextStamp;
                }
            }
            ++nextStamp;
            if (n2 == n) {
                break;
            }
            continue;
        }
        this.nextStamp = nextStamp;
    }
    
    private void invalidateWeekFields() {
        if (this.stamp[4] != 1 && this.stamp[3] != 1) {
            return;
        }
        Calendar calendar = (Calendar)this.clone();
        calendar.setLenient(true);
        calendar.clear(4);
        calendar.clear(3);
        if (this.stamp[4] == 1) {
            int value = calendar.get(4);
            if (this.fields[4] != value) {
                this.fields[4] = value;
            }
        }
        if (this.stamp[3] == 1) {
            int value2 = calendar.get(3);
            if (this.fields[3] != value2) {
                this.fields[3] = value2;
            }
        }
        return;
    }
    
    private synchronized void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (!this.isTimeSet) {
            try {
                this.updateTime();
            }
            catch (IllegalArgumentException ex) {}
        }
        TimeZone zone = null;
        if ((this.zone instanceof ZoneInfo)) {
            SimpleTimeZone lastRuleInstance = ((ZoneInfo)this.zone).getLastRuleInstance();
            if (lastRuleInstance == null) {
                lastRuleInstance = new SimpleTimeZone(this.zone.getRawOffset(), this.zone.getID());
            }
            zone = this.zone;
            this.zone = lastRuleInstance;
        }
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeObject(zone);
        if (zone != null) {
            this.zone = zone;
        }
    }
    
    private void readObject(final ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.stamp = new int[17];
        if (this.serialVersionOnStream >= 2) {
            this.isTimeSet = true;
            if (this.fields == null) {
                this.fields = new int[17];
            }
            if (this.isSet == null) {
                this.isSet = new boolean[17];
            }
        }
        else if (this.serialVersionOnStream >= 0) {
            for (int i = 0; i < 17; ++i) {
                this.stamp[i] = this.isSet[i];
            }
        }
        this.serialVersionOnStream = 1;
        ZoneInfo zone = null;
        try {
            zone = (ZoneInfo)AccessController.doPrivileged(new PrivilegedExceptionAction<ZoneInfo>() {
                @Override
                public ZoneInfo run() throws Exception {
                    return (ZoneInfo)objectInputStream.readObject();
                }
            }, Calendar.CalendarAccessControlContext.INSTANCE);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = ex.getException();
            if (!(exception instanceof OptionalDataException)) {
                if ((exception instanceof RuntimeException)) {
                    throw (RuntimeException)exception;
                }
                if ((exception instanceof IOException)) {
                    throw (IOException)exception;
                }
                if ((exception instanceof ClassNotFoundException)) {
                    throw (ClassNotFoundException)exception;
                }
                throw new RuntimeException(exception);
            }
        }
        if (zone != null) {
            this.zone = zone;
        }
        if ((this.zone instanceof SimpleTimeZone)) {
            String id = this.zone.getID();
            TimeZone timeZone = TimeZone.getTimeZone(id);
            if (timeZone != null && timeZone.hasSameRules(this.zone) && timeZone.getID().equals(id)) {
                this.zone = timeZone;
            }
        }
    }
    
    public final Instant toInstant() {
        return Instant.ofEpochMilli(this.getTimeInMillis());
    }
    
    static {
        cachedLocaleData = new ConcurrentHashMap(3);
        FIELD_NAME = new String[] { "ERA", "YEAR", "MONTH", "WEEK_OF_YEAR", "WEEK_OF_MONTH", "DAY_OF_MONTH", "DAY_OF_YEAR", "DAY_OF_WEEK", "DAY_OF_WEEK_IN_MONTH", "AM_PM", "HOUR", "HOUR_OF_DAY", "MINUTE", "SECOND", "MILLISECOND", "ZONE_OFFSET", "DST_OFFSET" };
    }
    
    public static class Builder
    {
        private static final int NFIELDS = 18;
        private static final int WEEK_YEAR = 17;
        private long instant;
        private int[] fields;
        private int nextStamp;
        private int maxFieldIndex;
        private String type;
        private TimeZone zone;
        private boolean lenient;
        private Locale locale;
        private int firstDayOfWeek;
        private int minimalDaysInFirstWeek;
        
        public Builder() {
            this.lenient = true;
        }
        
        public Calendar.Builder setInstant(long instant) {
            if (this.fields != null) {
                throw new IllegalStateException();
            }
            this.instant = instant;
            this.nextStamp = 1;
            return this;
        }
        
        public Calendar.Builder setInstant(Date date) {
            return this.setInstant(date.getTime());
        }
        
        public Calendar.Builder set(int n, int n2) {
            if (n < 0 || n >= 17) {
                throw new IllegalArgumentException("field is invalid");
            }
            if (this.isInstantSet()) {
                throw new IllegalStateException("instant has been set");
            }
            this.allocateFields();
            this.internalSet(n, n2);
            return this;
        }
        
        public Calendar.Builder setFields(int... array) {
            int length = array.length;
            if (length % 2 != 0) {
                throw new IllegalArgumentException();
            }
            if (this.isInstantSet()) {
                throw new IllegalStateException("instant has been set");
            }
            if (this.nextStamp + length / 2 < 0) {
                throw new IllegalStateException("stamp counter overflow");
            }
            this.allocateFields();
            int i = 0;
            while (i < length) {
                int n = array[i++];
                if (n < 0 || n >= 17) {
                    throw new IllegalArgumentException("field is invalid");
                }
                this.internalSet(n, array[i++]);
            }
            return this;
        }
        
        public Calendar.Builder setDate(int n, int n2, int n3) {
            return this.setFields(new int[] { 1, n, 2, n2, 5, n3 });
        }
        
        public Calendar.Builder setTimeOfDay(int n, int n2, int n3) {
            return this.setTimeOfDay(n, n2, n3, 0);
        }
        
        public Calendar.Builder setTimeOfDay(int n, int n2, int n3, int n4) {
            return this.setFields(new int[] { 11, n, 12, n2, 13, n3, 14, n4 });
        }
        
        public Calendar.Builder setWeekDate(int n, int n2, int n3) {
            this.allocateFields();
            this.internalSet(17, n);
            this.internalSet(3, n2);
            this.internalSet(7, n3);
            return this;
        }
        
        public Calendar.Builder setTimeZone(TimeZone zone) {
            if (zone == null) {
                throw new NullPointerException();
            }
            this.zone = zone;
            return this;
        }
        
        public Calendar.Builder setLenient(boolean lenient) {
            this.lenient = lenient;
            return this;
        }
        
        public Calendar.Builder setCalendarType(String str) {
            if (str.equals("gregorian")) {
                str = "gregory";
            }
            if (!Calendar.getAvailableCalendarTypes().contains(str) && !str.equals("iso8601")) {
                throw new IllegalArgumentException("unknown calendar type: " + str);
            }
            if (this.type == null) {
                this.type = str;
            }
            else if (!this.type.equals(str)) {
                throw new IllegalStateException("calendar type override");
            }
            return this;
        }
        
        public Calendar.Builder setLocale(Locale locale) {
            if (locale == null) {
                throw new NullPointerException();
            }
            this.locale = locale;
            return this;
        }
        
        public Calendar.Builder setWeekDefinition(int firstDayOfWeek, int minimalDaysInFirstWeek) {
            if (!this.isValidWeekParameter(firstDayOfWeek) || !this.isValidWeekParameter(minimalDaysInFirstWeek)) {
                throw new IllegalArgumentException();
            }
            this.firstDayOfWeek = firstDayOfWeek;
            this.minimalDaysInFirstWeek = minimalDaysInFirstWeek;
            return this;
        }
        
        public Calendar build() {
            if (this.locale == null) {
                this.locale = Locale.getDefault();
            }
            if (this.zone == null) {
                this.zone = TimeZone.getDefault();
            }
            if (this.type == null) {
                this.type = this.locale.getUnicodeLocaleType("ca");
            }
            if (this.type == null) {
                if (this.locale.getCountry() == "TH" && this.locale.getLanguage() == "th") {
                    this.type = "buddhist";
                }
                else {
                    this.type = "gregory";
                }
            }
            String type = this.type;
            int n = -1;
            switch (type.hashCode()) {
                case 283776265: {
                    if (type.equals("gregory")) {
                        n = 0;
                        break;
                    }
                    break;
                }
                case 2095190916: {
                    if (type.equals("iso8601")) {
                        n = 1;
                        break;
                    }
                    break;
                }
                case -1581060683: {
                    if (type.equals("buddhist")) {
                        n = 2;
                        break;
                    }
                    break;
                }
                case -752730191: {
                    if (type.equals("japanese")) {
                        n = 3;
                        break;
                    }
                    break;
                }
            }
            GregorianCalendar gregorianCalendar;
            switch (n) {
                case 0: {
                    gregorianCalendar = new GregorianCalendar(this.zone, this.locale, true);
                    break;
                }
                case 1: {
                    GregorianCalendar gregorianCalendar2 = new GregorianCalendar(this.zone, this.locale, true);
                    gregorianCalendar2.setGregorianChange(new Date(Long.MIN_VALUE));
                    this.setWeekDefinition(2, 4);
                    gregorianCalendar = gregorianCalendar2;
                    break;
                }
                case 2: {
                    gregorianCalendar = new BuddhistCalendar(this.zone, this.locale);
                    gregorianCalendar.clear();
                    break;
                }
                case 3: {
                    gregorianCalendar = new JapaneseImperialCalendar(this.zone, this.locale, true);
                    break;
                }
                default: {
                    throw new IllegalArgumentException("unknown calendar type: " + this.type);
                }
            }
            gregorianCalendar.setLenient(this.lenient);
            if (this.firstDayOfWeek != 0) {
                gregorianCalendar.setFirstDayOfWeek(this.firstDayOfWeek);
                gregorianCalendar.setMinimalDaysInFirstWeek(this.minimalDaysInFirstWeek);
            }
            if (this.isInstantSet()) {
                gregorianCalendar.setTimeInMillis(this.instant);
                gregorianCalendar.complete();
                return gregorianCalendar;
            }
            if (this.fields != null) {
                int n2 = this.isSet(17) && this.fields[17] > this.fields[1];
                if (n2 && !gregorianCalendar.isWeekDateSupported()) {
                    throw new IllegalArgumentException("week date is unsupported by " + this.type);
                }
                for (int i = 2; i < this.nextStamp; ++i) {
                    int j = 0;
                    while (j <= this.maxFieldIndex) {
                        if (this.fields[j] == i) {
                            gregorianCalendar.set(j, this.fields[18 + j]);
                            break;
                        }
                        ++j;
                    }
                }
                if (!(n2 == 0)) {
                    gregorianCalendar.setWeekDate(this.fields[35], this.isSet(3) ? this.fields[21] : 1, this.isSet(7) ? this.fields[25] : gregorianCalendar.getFirstDayOfWeek());
                }
                gregorianCalendar.complete();
            }
            return gregorianCalendar;
        }
        
        private void allocateFields() {
            if (this.fields == null) {
                this.fields = new int[36];
                this.nextStamp = 2;
                this.maxFieldIndex = -1;
            }
        }
        
        private void internalSet(int maxFieldIndex, int n) {
            this.fields[maxFieldIndex] = this.nextStamp++;
            if (this.nextStamp < 0) {
                throw new IllegalStateException("stamp counter overflow");
            }
            this.fields[18 + maxFieldIndex] = n;
            if (maxFieldIndex > this.maxFieldIndex && maxFieldIndex < 17) {
                this.maxFieldIndex = maxFieldIndex;
            }
            return;
        }
        
        private boolean isInstantSet() {
            return this.nextStamp == 1;
        }
        
        private boolean isSet(int n) {
            return this.fields != null && this.fields[n] > 0;
        }
        
        private boolean isValidWeekParameter(int n) {
            return n > 0 && n <= 7;
        }
    }
    
    private static class AvailableCalendarTypes
    {
        private static final Set<String> SET;
        
        static {
            HashSet set = new HashSet(3);
            set.add("gregory");
            set.add("buddhist");
            set.add("japanese");
            SET = Collections.unmodifiableSet(set);
        }
    }
    
    private static class CalendarAccessControlContext
    {
        private static final AccessControlContext INSTANCE;
        
        static {
            RuntimePermission runtimePermission = new RuntimePermission("accessClassInPackage.sun.util.calendar");
            PermissionCollection permissionCollection = runtimePermission.newPermissionCollection();
            permissionCollection.add(runtimePermission);
            INSTANCE = new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, permissionCollection) });
        }
    }
}
